package example;

import java.util.*;
import java.lang.reflect.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.geometry.ColorCube;
import java.util.logging.Level;
import java.util.logging.Logger;


// http://github.com/philjord/java3d-examples/blob/master/src/main/java/org/jdesktop/j3d/examples/hello_universe/HelloUniverse.java
// http://community.oracle.com/thread/1274674
// http://www.java2s.com/Code/Java/3D/Simple3DDemo.htm
// java3d-1_5_1-windows-amd64.exe

public class Window extends javax.swing.JFrame {

    private SimpleUniverse univ;
    private Canvas3D canvas;
    private BranchGroup scene;

    public Window() {
        initComponents();

        univ = new SimpleUniverse();        
        univ.getViewingPlatform().setNominalViewingTransform();
       
        canvas = univ.getCanvas();
	drawingPanel.add (canvas, java.awt.BorderLayout.CENTER);
        // drawingPanel with BorderLayout

        // scene = new BranchGroup();
        // scene.addChild(new ColorCube(0.3));
        scene = createSceneGraph();
        univ.addBranchGraph(scene);
        
        displayTree (scene);
    } 

    public BranchGroup createSceneGraph() 
    {
	// Create the root of the branch graph
	BranchGroup objRoot = new BranchGroup();
        

        // Create the TransformGroup node and initialize it to the
	// identity. Enable the TRANSFORM_WRITE capability so that
	// our behavior code can modify it at run time. Add it to
	// the root of the subgraph.
	TransformGroup objTrans = new TransformGroup();
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	objRoot.addChild(objTrans);

	// Create a simple Shape3D node; add it to the scene graph.
	objTrans.addChild(new ColorCube(0.4));

	// Create a new Behavior object that will perform the
	// desired operation on the specified transform and add
	// it into the scene graph.
	Transform3D yAxis = new Transform3D();
	Alpha rotationAlpha = new Alpha(-1, 4000);

	RotationInterpolator rotator =
	    new RotationInterpolator(rotationAlpha, objTrans, yAxis,
				     0.0f, (float) Math.PI*2.0f);
	BoundingSphere bounds =
	    new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);
	rotator.setSchedulingBounds(bounds);
	objRoot.addChild(rotator);
        
        // Have Java 3D perform optimizations on this scene graph.
        objRoot.compile();

	return objRoot;
    }


    /* tree */

    private void run ()
    {
       displayTree (scene);
    }
    
    DefaultTreeModel m; 
    
    private void displayTree (Object obj)
    {
       DefaultMutableTreeNode root = displayBranch (obj) ;
       m = new DefaultTreeModel (root);
       tree.setModel (m);
    }

    private DefaultMutableTreeNode displayBranch (Object obj)
    {   
       DefaultMutableTreeNode item = new DefaultMutableTreeNode (obj);
       if (obj instanceof SceneGraphObject)
       {
           SceneGraphObject gobj = (SceneGraphObject) obj;
           if (gobj instanceof Group)
           {
               Group grp = (Group) gobj;
               int cnt = grp.numChildren ();
               for (int inx = 0; inx < cnt; inx++)
               {
                   Node node = grp.getChild (inx);
                   item.add(displayBranch (node));
               }
           }
       }
       /*
       if (a.left != null)
             node.add (displayBranch (a.left)) ;
       if (a.middle != null)
             node.add (displayBranch (a.middle)) ;
       if (a.right != null)
             node.add (displayBranch (a.right)) ;
       */
       return item;
    } 
    
    /* table */
    
    DefaultTableModel tm;
    
    private void displayTable (Object obj)
    {
        
        Vector columns = new Vector ();
        columns.add ("Name");
        columns.add ("Value");
        
        Vector data = new Vector ();
 
        Class cls = obj.getClass();
        BeanInfo info;
        try 
        {
            info = Introspector.getBeanInfo (cls);
            for (PropertyDescriptor prop : info.getPropertyDescriptors() )
            {
               String name = prop.getName ();
               String value = "";

                try 
                {
                    Method method = prop.getReadMethod ();
                    if (method != null)
                    {
                        Object answer = method.invoke (obj);
                        if (answer != null)
                            value = answer.toString();
                    }
                }
                catch (IllegalArgumentException ex) { }
                catch (IllegalAccessException ex) { } 
                catch (InvocationTargetException ex) { }

               Vector line = new Vector ();
               line.add (name);
               line.add (value);
               data.add (line);
            }
        } 
        catch (IntrospectionException ex) { }
        
        tm = new DefaultTableModel (data, columns);
        table.setModel (tm);
    }
    
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        toolbar = new javax.swing.JPanel();
        runButton = new javax.swing.JButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        jSplitPane2 = new javax.swing.JSplitPane();
        jSplitPane3 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        tree = new javax.swing.JTree();
        drawingPanel = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();
        jScrollPane4 = new javax.swing.JScrollPane();
        output = new javax.swing.JTextArea();
        maniMenu = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        runMenuItem = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        runButton.setText("Run");
        runButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout toolbarLayout = new javax.swing.GroupLayout(toolbar);
        toolbar.setLayout(toolbarLayout);
        toolbarLayout.setHorizontalGroup(
            toolbarLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(toolbarLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(runButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        toolbarLayout.setVerticalGroup(
            toolbarLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(toolbarLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(runButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jSplitPane1.setDividerLocation(400);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(1.0);

        jSplitPane2.setDividerLocation(600);
        jSplitPane2.setResizeWeight(1.0);

        jSplitPane3.setDividerLocation(200);

        tree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                treeValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(tree);

        jSplitPane3.setLeftComponent(jScrollPane1);

        drawingPanel.setLayout(new java.awt.BorderLayout());
        jSplitPane3.setRightComponent(drawingPanel);

        jSplitPane2.setLeftComponent(jSplitPane3);

        table.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane3.setViewportView(table);

        jSplitPane2.setRightComponent(jScrollPane3);

        jSplitPane1.setTopComponent(jSplitPane2);

        output.setColumns(20);
        output.setRows(5);
        jScrollPane4.setViewportView(output);

        jSplitPane1.setRightComponent(jScrollPane4);

        fileMenu.setText("File");

        runMenuItem.setText("Run");
        runMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(runMenuItem);

        maniMenu.add(fileMenu);

        setJMenuBar(maniMenu);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(toolbar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 800, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(toolbar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 528, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void runButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_runButtonActionPerformed
        run ();
    }//GEN-LAST:event_runButtonActionPerformed

    private void treeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_treeValueChanged
        TreePath[] paths = evt.getPaths();
        for (TreePath path : paths)
            if (evt.isAddedPath(path))
            {
                Object obj = path.getLastPathComponent();
                if (obj instanceof DefaultMutableTreeNode)
                {
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode) obj;
                    displayTable (node.getUserObject());
                }
            }
    }//GEN-LAST:event_treeValueChanged

    private void runMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_runMenuItemActionPerformed
        run ();
    }//GEN-LAST:event_runMenuItemActionPerformed

    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        /*
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(Window.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(Window.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(Window.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(Window.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        */
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new Window().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel drawingPanel;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JSplitPane jSplitPane2;
    private javax.swing.JSplitPane jSplitPane3;
    private javax.swing.JMenuBar maniMenu;
    private javax.swing.JTextArea output;
    private javax.swing.JButton runButton;
    private javax.swing.JMenu runMenuItem;
    private javax.swing.JTable table;
    private javax.swing.JPanel toolbar;
    private javax.swing.JTree tree;
    // End of variables declaration//GEN-END:variables
}
